@@ -1,4 +1,3 @@
-#include "pch.h"
 #include "engine.h"
 
 struct normal
@@ -7,45 +6,22 @@ struct normal
     vec surface;
 };
 
-struct nkey
-{
-    ivec v;
-
-    nkey() {}
-    nkey(const ivec &origin, const vvec &offset)
-     : v(((origin.x&~VVEC_INT_MASK)<<VVEC_FRAC) | offset.x,
-         ((origin.y&~VVEC_INT_MASK)<<VVEC_FRAC) | offset.y,
-         ((origin.z&~VVEC_INT_MASK)<<VVEC_FRAC) | offset.z)
-    {}            
-};
-
 struct nval
 {
-    int normals;
+    int flat, normals;
 
-    nval() : normals(-1) {}
+    nval() : flat(0), normals(-1) {}
 };
 
-static inline bool htcmp(const nkey &x, const nkey &y)
-{
-    return x.v == y.v;
-}
-
-static inline uint hthash(const nkey &k)
-{
-    return k.v.x^k.v.y^k.v.z;
-}
-
-hashtable<nkey, nval> normalgroups(1<<16);
+hashtable<vec, nval> normalgroups(1<<16);
 vector<normal> normals;
 
-VARF(lerpangle, 0, 44, 180, hdr.lerpangle = lerpangle);
+VARR(lerpangle, 0, 44, 180);
 
 static float lerpthreshold = 0;
 
-void addnormal(const ivec &origin, const vvec &offset, const vec &surface)
+static void addnormal(const vec &key, const vec &surface)
 {
-    nkey key(origin, offset);
     nval &val = normalgroups[key];
     normal &n = normals.add();
     n.next = val.normals;
@@ -53,14 +29,25 @@ void addnormal(const ivec &origin, const vvec &offset, const vec &surface)
     val.normals = normals.length()-1;
 }
 
-void findnormal(const ivec &origin, const vvec &offset, const vec &surface, vec &v)
+static void addnormal(const vec &key, int axis)
 {
-    nkey key(origin, offset);
-    nval *val = normalgroups.access(key);
+    nval &val = normalgroups[key];
+    val.flat += 1<<(4*axis);
+}
+
+void findnormal(const vec &key, const vec &surface, vec &v)
+{
+    const nval *val = normalgroups.access(key);
     if(!val) { v = surface; return; }
 
     v = vec(0, 0, 0);
     int total = 0;
+    if(surface.x >= lerpthreshold) { int n = (val->flat>>4)&0xF; v.x += n; total += n; }
+    else if(surface.x <= -lerpthreshold) { int n = val->flat&0xF; v.x -= n; total += n; }
+    if(surface.y >= lerpthreshold) { int n = (val->flat>>12)&0xF; v.y += n; total += n; }
+    else if(surface.y <= -lerpthreshold) { int n = (val->flat>>8)&0xF; v.y -= n; total += n; }
+    if(surface.z >= lerpthreshold) { int n = (val->flat>>20)&0xF; v.z += n; total += n; }
+    else if(surface.z <= -lerpthreshold) { int n = (val->flat>>16)&0xF; v.z -= n; total += n; }
     for(int cur = val->normals; cur >= 0;)
     {
         normal &o = normals[cur];
@@ -75,15 +62,15 @@ void findnormal(const ivec &origin, const vvec &offset, const vec &surface, vec
     else if(!total) v = surface;
 }
 
-VAR(lerpsubdiv, 0, 2, 4);
-VAR(lerpsubdivsize, 4, 4, 128);
+VARR(lerpsubdiv, 0, 2, 4);
+VARR(lerpsubdivsize, 4, 4, 128);
 
 static uint progress = 0;
 
 void show_calcnormals_progress()
 {
     float bar1 = float(progress) / float(allocnodes);
-    show_out_of_renderloop_progress(bar1, "computing normals...");
+    renderprogress(bar1, "computing normals...");
 }
 
 #define CHECK_PROGRESS(exit) CHECK_CALCLIGHT_PROGRESS(exit, show_calcnormals_progress)
@@ -101,19 +88,22 @@ void addnormals(cube &c, const ivec &o, int size)
     }
     else if(isempty(c)) return;
 
-    vvec vvecs[8];
-    bool usefaces[6];
-    int vertused = calcverts(c, o.x, o.y, o.z, size, vvecs, usefaces);
     vec verts[8];
-    loopi(8) if(vertused&(1<<i)) verts[i] = vvecs[i].tovec(o);
+    int vertused = 0, usefaces[6];
+    loopi(6) if((usefaces[i] = visibletris(c, i, o.x, o.y, o.z, size))) vertused |= fvmasks[1<<i];
+    loopi(8) if(vertused&(1<<i)) calcvert(c, o.x, o.y, o.z, size, verts[i], i);
     loopi(6) if(usefaces[i])
     {
         CHECK_PROGRESS(return);
         if(c.texture[i] == DEFAULT_SKY) continue;
 
         plane planes[2];
-        int numplanes = genclipplane(c, i, verts, planes);
-        if(!numplanes) continue;
+        int numplanes = 0;
+        if(!flataxisface(c, i))
+        {
+            numplanes = genclipplane(c, i, verts, planes);
+            if(!numplanes) continue;
+        }
         int subdiv = 0;
         if(lerpsubdiv && size > lerpsubdivsize) // && faceedges(c, i) == F_SOLID)
         {
@@ -131,19 +121,34 @@ void addnormals(cube &c, const ivec &o, int size)
         loopj(4) idxs[j] = faceverts(c, i, j);
         loopj(4)
         {
-            const vvec &v = vvecs[idxs[j]], &vn = vvecs[idxs[(j+1)%4]];
+            const vec &v = verts[idxs[j]], &vn = verts[idxs[(j+1)%4]];
             if(v==vn) continue;
+            if(!numplanes)
+            {
+                addnormal(v, i);
+                if(subdiv < 2) continue;
+                vec dv = vec(vn).sub(v);
+                if(dv.iszero()) continue;
+                dv.div(subdiv);
+                vec vs(v);
+                loopk(subdiv - 1)
+                {
+                    vs.add(dv);
+                    addnormal(vs, i);
+                }
+                continue;
+            }
             const vec &cur = numplanes < 2 || j == 1 ? planes[0] : (j == 3 ? planes[1] : avg);
-            addnormal(o, v, cur);
+            addnormal(v, cur);
             if(subdiv < 2) continue;
-            ivec dv;
-            loopk(3) dv[k] = (int(vn[k]) - int(v[k])) / subdiv;
+            vec dv = vec(vn).sub(v);
             if(dv.iszero()) continue;
-            vvec vs(v);
+            dv.div(subdiv);
+            vec vs(v);
             if(numplanes < 2) loopk(subdiv - 1)
             {
                 vs.add(dv);
-                addnormal(o, vs, planes[0]);
+                addnormal(vs, planes[0]);
             }
             else
             {
@@ -155,7 +160,7 @@ void addnormals(cube &c, const ivec &o, int size)
                 {
                     vs.add(dv);
                     n.add(dn);
-                    addnormal(o, vs, vec(dn).normalize());
+                    addnormal(vs, vec(dn).normalize());
                 }
             }
         }
@@ -167,13 +172,13 @@ void calcnormals()
     if(!lerpangle) return;
     lerpthreshold = cos(lerpangle*RAD) - 1e-5f; 
     progress = 1;
-    loopi(8) addnormals(worldroot[i], ivec(i, 0, 0, 0, hdr.worldsize/2), hdr.worldsize/2);
+    loopi(8) addnormals(worldroot[i], ivec(i, 0, 0, 0, worldsize/2), worldsize/2);
 }
 
 void clearnormals()
 {
     normalgroups.clear();
-    normals.setsizenodelete(0);
+    normals.setsize(0);
 }
 
 void calclerpverts(const vec &origin, const vec *p, const vec *n, const vec &ustep, const vec &vstep, lerpvert *lv, int &numv)
