@@ -1,6 +1,6 @@
 struct vertmodel : animmodel
 {
-    struct vert { vec norm, pos; };
+    struct vert { vec pos, norm; };
     struct vvertff { vec pos; float u, v; };
     struct vvert : vvertff { vec norm; };
     struct vvertbump : vvert { vec tangent; float bitangent; };
@@ -41,31 +41,78 @@ struct vertmodel : animmodel
             DELETEA(tris);
         }
 
-        virtual mesh *allocate() { return new vertmesh; }
-
-        mesh *copy()
+        void smoothnorms(float limit = 0, bool areaweight = true)
         {
-            vertmesh &m = *(vertmesh *)mesh::copy();
-            m.numverts = numverts;
-            m.verts = new vert[numverts*((vertmeshgroup *)group)->numframes];
-            memcpy(m.verts, verts, numverts*((vertmeshgroup *)group)->numframes*sizeof(vert));
-            m.tcverts = new tcvert[numverts];
-            memcpy(m.tcverts, tcverts, numverts*sizeof(tcvert));
-            m.numtris = numtris;
-            m.tris = new tri[numtris];
-            memcpy(m.tris, tris, numtris*sizeof(tri));
-            if(bumpverts)
+            if(((vertmeshgroup *)group)->numframes != 1)
             {
-                m.bumpverts = new bumpvert[numverts];
-                memcpy(m.bumpverts, bumpverts, numverts*sizeof(bumpvert));
+                buildnorms(areaweight);
+                return;
             }
-            else m.bumpverts = NULL;
-            return &m;
+            hashtable<vec, int> share;
+            int *next = new int[numverts];
+            memset(next, -1, numverts*sizeof(int));
+            loopi(numverts)
+            {
+                vert &v = verts[i];
+                v.norm = vec(0, 0, 0);
+                int idx = share.access(v.pos, i);
+                if(idx != i) { next[i] = next[idx]; next[idx] = i; }
+            }
+            loopi(numtris)
+            {
+                tri &t = tris[i];
+                vert &v1 = verts[t.vert[0]], &v2 = verts[t.vert[1]], &v3 = verts[t.vert[2]];
+                vec norm;
+                norm.cross(vec(v2.pos).sub(v1.pos), vec(v3.pos).sub(v1.pos));
+                if(!areaweight) norm.normalize();
+                v1.norm.add(norm);
+                v2.norm.add(norm);
+                v3.norm.add(norm);
+            }
+            vec *norms = new vec[numverts];
+            memset(norms, 0, numverts*sizeof(vec));
+            loopi(numverts)
+            {
+                vert &v = verts[i];
+                norms[i].add(v.norm);
+                if(next[i] >= 0)
+                {
+                    float vlimit = limit*v.norm.magnitude();
+                    for(int j = next[i]; j >= 0; j = next[j])
+                    {
+                        vert &o = verts[j];
+                        if(v.norm.dot(o.norm) >= vlimit*o.norm.magnitude())
+                        {
+                            norms[i].add(o.norm);
+                            norms[j].add(v.norm);
+                        }
+                    }
+                }
+            }
+            loopi(numverts) verts[i].norm = norms[i].normalize();
+            delete[] next;
+            delete[] norms;
         }
 
-        void scaleverts(const vec &transdiff, float scalediff)
+        void buildnorms(bool areaweight = true)
         {
-            loopi(((vertmeshgroup *)group)->numframes*numverts) verts[i].pos.add(transdiff).mul(scalediff);
+            loopk(((vertmeshgroup *)group)->numframes)
+            {
+                vert *fverts = &verts[k*numverts];
+                loopi(numverts) fverts[i].norm = vec(0, 0, 0);
+                loopi(numtris)
+                {
+                    tri &t = tris[i];
+                    vert &v1 = fverts[t.vert[0]], &v2 = fverts[t.vert[1]], &v3 = fverts[t.vert[2]];
+                    vec norm;
+                    norm.cross(vec(v2.pos).sub(v1.pos), vec(v3.pos).sub(v1.pos));
+                    if(!areaweight) norm.normalize();
+                    v1.norm.add(norm);
+                    v2.norm.add(norm);
+                    v3.norm.add(norm);
+                }
+                loopi(numverts) fverts[i].norm.normalize();
+            }
         }
 
         void calctangents()
@@ -138,7 +185,7 @@ struct vertmodel : animmodel
             loopj(numtris)
             {
                 BIH::tri &t = out[noclip ? 1 : 0].add();
-                t.tex = tex->bpp==32 ? tex : NULL;
+                t.tex = tex;
                 t.a = m.transform(fverts[tris[j].vert[0]].pos);
                 t.b = m.transform(fverts[tris[j].vert[1]].pos);
                 t.c = m.transform(fverts[tris[j].vert[2]].pos);
@@ -198,7 +245,7 @@ struct vertmodel : animmodel
         }
 
         template<class T>
-        int genvbo(vector<ushort> &idxs, int offset, vector<T> &vverts)
+        int genvbo(vector<ushort> &idxs, int offset, vector<T> &vverts, int *htdata, int htlen)
         {
             voffset = offset;
             eoffset = idxs.length();
@@ -211,13 +258,13 @@ struct vertmodel : animmodel
                     int index = t.vert[j];
                     tcvert &tc = tcverts[index];
                     vert &v = verts[index];
-                    loopvk(vverts)
+                    int htidx = hthash(v.pos)&(htlen-1);
+                    loopk(htlen)
                     {
-                        if(comparevert(vverts[k], index, tc, v)) { minvert = min(minvert, (ushort)k); idxs.add((ushort)k); goto found; }
+                        int &vidx = htdata[(htidx+k)&(htlen-1)];
+                        if(vidx < 0) { vidx = idxs.add(ushort(vverts.length())); assignvert(vverts.add(), index, tc, v); break; }
+                        else if(comparevert(vverts[vidx], index, tc, v)) { minvert = min(minvert, idxs.add(ushort(vidx))); break; }
                     }
-                    idxs.add(vverts.length());
-                    assignvert(vverts.add(), index, tc, v);
-                found:;
                 }
             }
             minvert = min(minvert, ushort(voffset));
@@ -251,61 +298,47 @@ struct vertmodel : animmodel
             }
         }
 
-        void interpverts(const animstate &as, bool norms, bool tangents, void *vdata, skin &s)
+        void interpverts(const animstate &as, bool norms, bool tangents, void * RESTRICT vdata, skin &s)
         {
-            vert *vert1 = &verts[as.cur.fr1 * numverts],
-                 *vert2 = &verts[as.cur.fr2 * numverts],
-                 *pvert1 = as.interp<1 ? &verts[as.prev.fr1 * numverts] : NULL, 
-                 *pvert2 = as.interp<1 ? &verts[as.prev.fr2 * numverts] : NULL;
-            #define ip(p1, p2, t)   (p1+t*(p2-p1))
-            #define ip_v(p, c, t)   ip(p##1[i].c, p##2[i].c, t)
-            #define ip_v_ai(c)      ip(ip_v(pvert, c, as.prev.t), ip_v(vert, c, as.cur.t), as.interp)
-            #define ip_pos          vec(ip_v(vert, pos.x, as.cur.t), ip_v(vert, pos.y, as.cur.t), ip_v(vert, pos.z, as.cur.t))
-            #define ip_pos_ai       vec(ip_v_ai(pos.x), ip_v_ai(pos.y), ip_v_ai(pos.z))
-            #define ip_norm         vec(ip_v(vert, norm.x, as.cur.t), ip_v(vert, norm.y, as.cur.t), ip_v(vert, norm.z, as.cur.t))
-            #define ip_norm_ai      vec(ip_v_ai(norm.x), ip_v_ai(norm.y), ip_v_ai(norm.z))
-            #define ip_b_ai(c)      ip(ip_v(bpvert, c, as.prev.t), ip_v(bvert, c, as.cur.t), as.interp)
-            #define ip_tangent      vec(ip_v(bvert, tangent.x, as.cur.t), ip_v(bvert, tangent.y, as.cur.t), ip_v(bvert, tangent.z, as.cur.t))
-            #define ip_tangent_ai   vec(ip_b_ai(tangent.x), ip_b_ai(tangent.y), ip_b_ai(tangent.z))
+            const vert * RESTRICT vert1 = &verts[as.cur.fr1 * numverts],
+                       * RESTRICT vert2 = &verts[as.cur.fr2 * numverts],
+                       * RESTRICT pvert1 = as.interp<1 ? &verts[as.prev.fr1 * numverts] : NULL, 
+                       * RESTRICT pvert2 = as.interp<1 ? &verts[as.prev.fr2 * numverts] : NULL;
+            #define ipvert(attrib)   v.attrib.lerp(vert1[i].attrib, vert2[i].attrib, as.cur.t)
+            #define ipbvert(attrib)  v.attrib.lerp(bvert1[i].attrib, bvert2[i].attrib, as.cur.t)
+            #define ipvertp(attrib)  v.attrib.lerp(pvert1[i].attrib, pvert2[i].attrib, as.prev.t).lerp(vec().lerp(vert1[i].attrib, vert2[i].attrib, as.cur.t), as.interp)
+            #define ipbvertp(attrib) v.attrib.lerp(bpvert1[i].attrib, bpvert2[i].attrib, as.prev.t).lerp(vec().lerp(bvert1[i].attrib, bvert2[i].attrib, as.cur.t), as.interp)
             #define iploop(type, body) \
                 loopi(numverts) \
                 { \
-                    type &v = ((type *)vdata)[i]; \
+                    type &v = ((type * RESTRICT)vdata)[i]; \
                     body; \
                 }
             if(tangents)
             {
-                bumpvert *bvert1 = &bumpverts[as.cur.fr1 * numverts],
-                         *bvert2 = &bumpverts[as.cur.fr2 * numverts],
-                         *bpvert1 = as.interp<1 ? &bumpverts[as.prev.fr1 * numverts] : NULL, 
-                         *bpvert2 = as.interp<1 ? &bumpverts[as.prev.fr2 * numverts] : NULL;
-                if(as.interp<1) iploop(vvertbump, { v.pos = ip_pos_ai; v.norm = ip_norm_ai; v.tangent = ip_tangent_ai; v.bitangent = bvert1[i].bitangent; })
-                else iploop(vvertbump, { v.pos = ip_pos; v.norm = ip_norm; v.tangent = ip_tangent; v.bitangent = bvert1[i].bitangent; })
+                const bumpvert * RESTRICT bvert1 = &bumpverts[as.cur.fr1 * numverts],
+                               * RESTRICT bvert2 = &bumpverts[as.cur.fr2 * numverts],
+                               * RESTRICT bpvert1 = as.interp<1 ? &bumpverts[as.prev.fr1 * numverts] : NULL, 
+                               * RESTRICT bpvert2 = as.interp<1 ? &bumpverts[as.prev.fr2 * numverts] : NULL;
+                if(as.interp<1) iploop(vvertbump, { ipvertp(pos); ipvertp(norm); ipbvertp(tangent); v.bitangent = bvert1[i].bitangent; })
+                else iploop(vvertbump, { ipvert(pos); ipvert(norm); ipbvert(tangent); v.bitangent = bvert1[i].bitangent; })
             }
             else if(norms)
             {
-                if(as.interp<1) iploop(vvert, { v.pos = ip_pos_ai; v.norm = ip_norm_ai; })
-                else iploop(vvert, { v.pos = ip_pos; v.norm = ip_norm; })
+                if(as.interp<1) iploop(vvert, { ipvertp(pos); ipvertp(norm); })
+                else iploop(vvert, { ipvert(pos); ipvert(norm); })
             }
-            else if(as.interp<1) iploop(vvertff, v.pos = ip_pos_ai)
-            else iploop(vvertff, v.pos = ip_pos)
+            else if(as.interp<1) iploop(vvertff, ipvertp(pos))
+            else iploop(vvertff, ipvert(pos))
             #undef iploop
-            #undef ip
-            #undef ip_v
-            #undef ip_v_ai
-            #undef ip_pos
-            #undef ip_pos_ai
-            #undef ip_norm
-            #undef ip_norm_ai
-            #undef ip_b_ai
-            #undef ip_tangent
-            #undef ip_tangent_ai
+            #undef ipvert
+            #undef ipbvert
+            #undef ipvertp
+            #undef ipbvertp
         }
 
         void render(const animstate *as, skin &s, vbocacheentry &vc)
         {
-            s.bind(this, as);
-
             if(!(as->anim&ANIM_NOSKIN))
             {
                 if(s.multitextured())
@@ -328,15 +361,15 @@ struct vertmodel : animmodel
 
                 if(s.tangents())
                 {
-                    if(!enabletangents || lastnbuf!=lastvbuf)
+                    if(!enabletangents || lastxbuf!=lastvbuf)
                     {
                         if(!enabletangents) glEnableVertexAttribArray_(1);
-                        if(lastnbuf!=lastvbuf)
+                        if(lastxbuf!=lastvbuf)
                         {
                             vvertbump *vverts = hasVBO ? 0 : (vvertbump *)vc.vdata;
                             glVertexAttribPointer_(1, 4, GL_FLOAT, GL_FALSE, ((vertmeshgroup *)group)->vertsize, &vverts->tangent.x);
                         }
-                        lastnbuf = lastvbuf;
+                        lastxbuf = lastvbuf;
                         enabletangents = true;
                     }
                 }
@@ -424,64 +457,32 @@ struct vertmodel : animmodel
             return -1;
         }
 
-        virtual meshgroup *allocate() { return new vertmeshgroup; }
-
-        meshgroup *copy()
-        {
-            vertmeshgroup &group = *(vertmeshgroup *)meshgroup::copy();
-            group.numframes = numframes;
-            group.numtags = numtags;
-            group.tags = new tag[numframes*numtags];
-            memcpy(group.tags, tags, numframes*numtags*sizeof(tag));
-            loopi(numframes*numtags) if(group.tags[i].name) group.tags[i].name = newstring(group.tags[i].name);
-            return &group;
-        }
-        
         int totalframes() const { return numframes; }
 
-        void scaletags(const vec &transdiff, float scalediff)
-        {
-            loopi(numframes*numtags) 
-            {
-                matrix3x4 &m = tags[i].transform;
-                m.X.w = (m.X.w+transdiff.x)*scalediff;
-                m.Y.w = (m.Y.w+transdiff.y)*scalediff;
-                m.Z.w = (m.Z.w+transdiff.z)*scalediff;
-            }
-        }
-
-        void concattagtransform(int frame, int i, const matrix3x4 &m, matrix3x4 &n)
+        void concattagtransform(part *p, int frame, int i, const matrix3x4 &m, matrix3x4 &n)
         {
             n.mul(m, tags[frame*numtags + i].transform);
+            n.translate(m.transformnormal(p->translate).mul(p->model->scale));
         }
 
-        void calctagmatrix(int i, const animstate &as, GLfloat *matrix)
+        void calctagmatrix(part *p, int i, const animstate &as, glmatrixf &matrix)
         {
             const matrix3x4 &tag1 = tags[as.cur.fr1*numtags + i].transform, 
                             &tag2 = tags[as.cur.fr2*numtags + i].transform;
-            #define ip(p1, p2, t) (p1+t*(p2-p1))
-            #define ip_ai_tag(c) ip( ip( tag1p.c, tag2p.c, as.prev.t), ip( tag1.c, tag2.c, as.cur.t), as.interp)
+            matrix3x4 tag;
+            tag.lerp(tag1, tag2, as.cur.t);
             if(as.interp<1)
             {
                 const matrix3x4 &tag1p = tags[as.prev.fr1*numtags + i].transform, 
                                 &tag2p = tags[as.prev.fr2*numtags + i].transform;
-                loopj(4)
-                {
-                    matrix[4*j+0] = ip_ai_tag(X[j]);
-                    matrix[4*j+1] = ip_ai_tag(Y[j]);
-                    matrix[4*j+2] = ip_ai_tag(Z[j]);
-                }
+                matrix3x4 tagp;
+                tagp.lerp(tag1p, tag2p, as.prev.t);
+                tag.lerp(tagp, tag, as.interp);
             }
-            else loopj(4)
-            {
-                matrix[4*j+0] = ip(tag1.X[j], tag2.X[j], as.cur.t);
-                matrix[4*j+1] = ip(tag1.Y[j], tag2.Y[j], as.cur.t);
-                matrix[4*j+2] = ip(tag1.Z[j], tag2.Z[j], as.cur.t);
-            } 
-            #undef ip_ai_tag
-            #undef ip 
-            matrix[3] = matrix[7] = matrix[11] = 0.0f;
-            matrix[15] = 1.0f;
+            matrix = glmatrixf(tag);
+            matrix[12] = (matrix[12] + p->translate.x) * p->model->scale;
+            matrix[13] = (matrix[13] + p->translate.y) * p->model->scale;
+            matrix[14] = (matrix[14] + p->translate.z) * p->model->scale;
         }
 
         void genvbo(bool norms, bool tangents, vbocacheentry &vc)
@@ -524,7 +525,7 @@ struct vertmodel : animmodel
                     do \
                     { \
                         vector<type> vverts; \
-                        loopv(meshes) vlen += ((vertmesh *)meshes[i])->genvbo(idxs, vlen, vverts); \
+                        loopv(meshes) vlen += ((vertmesh *)meshes[i])->genvbo(idxs, vlen, vverts, htdata, htlen); \
                         if(hasVBO) glBufferData_(GL_ARRAY_BUFFER_ARB, vverts.length()*sizeof(type), vverts.getbuf(), GL_STATIC_DRAW_ARB); \
                         else \
                         { \
@@ -533,9 +534,17 @@ struct vertmodel : animmodel
                             memcpy(vc.vdata, vverts.getbuf(), vverts.length()*sizeof(type)); \
                         } \
                     } while(0)
+                int numverts = 0, htlen = 128;
+                loopv(meshes) numverts += ((vertmesh *)meshes[i])->numverts;
+                while(htlen < numverts) htlen *= 2;
+                if(numverts*4 > htlen*3) htlen *= 2; 
+                int *htdata = new int[htlen];
+                memset(htdata, -1, htlen*sizeof(int));
                 if(tangents) GENVBO(vvertbump);
                 else if(norms) GENVBO(vvert);
                 else GENVBO(vvertff);
+                delete[] htdata;
+                if(hasVBO) glBindBuffer_(GL_ARRAY_BUFFER_ARB, 0);
             }
 
             if(hasVBO)
@@ -543,6 +552,7 @@ struct vertmodel : animmodel
                 glGenBuffers_(1, &ebuf);
                 glBindBuffer_(GL_ELEMENT_ARRAY_BUFFER_ARB, ebuf);
                 glBufferData_(GL_ELEMENT_ARRAY_BUFFER_ARB, idxs.length()*sizeof(ushort), idxs.getbuf(), GL_STATIC_DRAW_ARB);
+                glBindBuffer_(GL_ELEMENT_ARRAY_BUFFER_ARB, 0);
             }
             else
             {
@@ -566,24 +576,42 @@ struct vertmodel : animmodel
                 if(hasVBO) glBindBuffer_(GL_ARRAY_BUFFER_ARB, vc.vbuf);
                 if(!lastvbuf) glEnableClientState(GL_VERTEX_ARRAY);
                 glVertexPointer(3, GL_FLOAT, vertsize, &vverts->pos);
+                lastvbuf = hasVBO ? (void *)(size_t)vc.vbuf : vc.vdata;
             }
-            lastvbuf = hasVBO ? (void *)(size_t)vc.vbuf : vc.vdata;
             if(as->anim&ANIM_NOSKIN)
             {
                 if(enabletc) disabletc();
+                if(enablenormals) disablenormals();
             }
-            else if(!enabletc || lasttcbuf!=lastvbuf)
+            else
             {
                 if(vnorms || vtangents)
                 {
-                    if(!enabletc) glEnableClientState(GL_NORMAL_ARRAY);
-                    if(lasttcbuf!=lastvbuf) glNormalPointer(GL_FLOAT, vertsize, &vverts->norm);
+                    if(!enablenormals)
+                    {
+                        glEnableClientState(GL_NORMAL_ARRAY);
+                        enablenormals = true;
+                    }
+                    if(lastnbuf!=lastvbuf)
+                    {
+                        glNormalPointer(GL_FLOAT, vertsize, &vverts->norm);
+                        lastnbuf = lastvbuf;
+                    }
+                }
+                else if(enablenormals) disablenormals();
+
+                if(!enabletc)
+                {
+                    glEnableClientState(GL_TEXTURE_COORD_ARRAY);
+                    enabletc = true;
+                }
+                if(lasttcbuf!=lastvbuf)
+                {
+                    glTexCoordPointer(2, GL_FLOAT, vertsize, &vverts->u);
+                    lasttcbuf = lastnbuf;
                 }
-                if(!enabletc) glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-                if(lasttcbuf!=lastvbuf) glTexCoordPointer(2, GL_FLOAT, vertsize, &vverts->u);
-                lasttcbuf = lastvbuf;
-                enabletc = true;
             }
+            if(enablebones) disablebones();
         }
 
         void cleanup()
@@ -597,19 +625,23 @@ struct vertmodel : animmodel
             }
             if(hasVBO) { if(ebuf) { glDeleteBuffers_(1, &ebuf); ebuf = 0; } }
             else DELETEA(vdata);
-            lastvbuf = lasttcbuf = lastmtcbuf = lastnbuf = NULL;
-            lastebuf = 0;
         }
 
-        void render(const animstate *as, float pitch, const vec &axis, part *p)
+        void render(const animstate *as, float pitch, const vec &axis, dynent *d, part *p)
         {
+            if(as->anim&ANIM_NORENDER)
+            {
+                loopv(p->links) calctagmatrix(p, p->links[i].tag, *as, p->links[i].matrix);
+                return;
+            }
+
             bool norms = false, tangents = false;
             loopv(p->skins) 
             {
                 if(p->skins[i].normals()) norms = true;
                 if(p->skins[i].tangents()) tangents = true;
             }
-            if(norms!=vnorms || tangents!=vtangents) cleanup();
+            if(norms!=vnorms || tangents!=vtangents) { cleanup(); disablevbo(); }
             vbocacheentry *vc = NULL;
             if(numframes<=1) vc = vbocache;
             else
@@ -644,9 +676,14 @@ struct vertmodel : animmodel
             }
         
             bindvbo(as, *vc);
-            loopv(meshes) ((vertmesh *)meshes[i])->render(as, p->skins[i], *vc);
+            loopv(meshes)
+            {
+                vertmesh *m = (vertmesh *)meshes[i];
+                p->skins[i].bind(m, as);
+                m->render(as, p->skins[i], *vc);
+            }
             
-            loopv(p->links) calctagmatrix(p->links[i].tag, *as, p->links[i].matrix);
+            loopv(p->links) calctagmatrix(p, p->links[i].tag, *as, p->links[i].matrix);
         }
     };
 
