@@ -13,15 +13,6 @@ struct entity                                   // persistent map entity
     uchar reserved;
 };
 
-enum
-{
-    TRIGGER_RESET = 0,
-    TRIGGERING,
-    TRIGGERED,
-    TRIGGER_RESETTING,
-    TRIGGER_DISAPPEARED
-};
-
 struct entitylight
 {
     vec color, dir;
@@ -32,12 +23,19 @@ struct entitylight
 
 struct extentity : entity                       // part of the entity that doesn't get saved to disk
 {
-    uchar spawned, inoctanode, visible, triggerstate;        // the only dynamic state of a map entity
+    enum
+    {
+        F_NOVIS     = 1<<0,
+        F_NOSHADOW  = 1<<1,
+        F_NOCOLLIDE = 1<<2,
+        F_ANIM      = 1<<3
+    };
+
+    uchar spawned, inoctanode, visible, flags;  // the only dynamic state of a map entity
     entitylight light;
-    int lasttrigger;
     extentity *attached;
 
-    extentity() : visible(false), triggerstate(TRIGGER_RESET), lasttrigger(0), attached(NULL) {}
+    extentity() : visible(false), flags(0), attached(NULL) {}
 };
 
 #define MAXENTS 10000
@@ -50,7 +48,7 @@ enum { PHYS_FLOAT = 0, PHYS_FALL, PHYS_SLIDE, PHYS_SLOPE, PHYS_FLOOR, PHYS_STEP_
 
 enum { ENT_PLAYER = 0, ENT_AI, ENT_INANIMATE, ENT_CAMERA, ENT_BOUNCE };
 
-enum { COLLIDE_AABB = 0, COLLIDE_ELLIPSE };
+enum { COLLIDE_AABB = 0, COLLIDE_OBB, COLLIDE_ELLIPSE };
 
 struct physent                                  // base entity type, can be affected by physics
 {
@@ -72,16 +70,13 @@ struct physent                                  // base entity type, can be affe
     uchar type;                                 // one of ENT_* above
     uchar collidetype;                          // one of COLLIDE_* above           
 
-    bool blocked, moving;                       // used by physics to signal ai
-    physent *onplayer;
-    int lastmove, lastmoveattempt, collisions, stacks;
+    bool blocked;                               // used by physics to signal ai
 
-    physent() : o(0, 0, 0), deltapos(0, 0, 0), newpos(0, 0, 0), yaw(270), pitch(0), roll(0), maxspeed(100), 
+    physent() : o(0, 0, 0), deltapos(0, 0, 0), newpos(0, 0, 0), yaw(0), pitch(0), roll(0), maxspeed(100), 
                radius(4.1f), eyeheight(14), aboveeye(1), xradius(4.1f), yradius(4.1f), zmargin(0),
                state(CS_ALIVE), editstate(CS_ALIVE), type(ENT_PLAYER),
                collidetype(COLLIDE_ELLIPSE),
-               blocked(false), moving(true),
-               onplayer(NULL), lastmove(0), lastmoveattempt(0), collisions(0), stacks(0)
+               blocked(false)
                { reset(); }
               
     void resetinterp()
@@ -102,6 +97,8 @@ struct physent                                  // base entity type, can be affe
 
     vec feetpos(float offset = 0) const { return vec(o).add(vec(0, 0, offset - eyeheight)); }
     vec headpos(float offset = 0) const { return vec(o).add(vec(0, 0, offset)); }
+
+    bool maymove() const { return timeinair || physstate < PHYS_FLOOR || vel.squaredlen() > 1e-4f || deltapos.squaredlen() > 1e-4f; } 
 };
 
 enum
@@ -119,6 +116,20 @@ enum
     NUMANIMS
 };
 
+static const char * const animnames[] =
+{
+    "dead", "dying", "idle",
+    "forward", "backward", "left", "right",
+    "hold 1", "hold 2", "hold 3", "hold 4", "hold 5", "hold 6", "hold 7",
+    "attack 1", "attack 2", "attack 3", "attack 4", "attack 5", "attack 6", "attack 7",
+    "pain",
+    "jump", "sink", "swim",
+    "edit", "lag", "taunt", "win", "lose",
+    "gun idle", "gun shoot",
+    "vwep idle", "vwep shoot", "shield", "powerup",
+    "mapmodel", "trigger"
+};
+
 #define ANIM_ALL         0x7F
 #define ANIM_INDEX       0x7F
 #define ANIM_LOOP        (1<<7)
@@ -134,6 +145,7 @@ enum
 #define ANIM_NORENDER    (1<<26)
 #define ANIM_RAGDOLL     (1<<27)
 #define ANIM_SETSPEED    (1<<28)
+#define ANIM_GHOST       (1<<29)
 #define ANIM_FLAGS       (0x1FF<<22)
 
 struct animinfo // description of a character's animation
@@ -157,7 +169,7 @@ struct animinterpinfo // used for animation blending of animated characters
     animinterpinfo() : lastswitch(-1), lastmodel(NULL) {}
 };
 
-#define MAXANIMPARTS 2
+#define MAXANIMPARTS 3
 
 struct occludequery;
 struct ragdolldata;
@@ -165,7 +177,6 @@ struct ragdolldata;
 struct dynent : physent                         // animated characters, or characters that can receive input
 {
     bool k_left, k_right, k_up, k_down;         // see input code
-    float targetyaw, rotspeed;                  // AI rotation
 
     entitylight light;
     animinterpinfo animinterp[MAXANIMPARTS];
@@ -190,7 +201,6 @@ struct dynent : physent                         // animated characters, or chara
     {
         k_left = k_right = k_up = k_down = jumping = false;
         move = strafe = 0;
-        targetyaw = rotspeed = 0;
     }
         
     void reset()
@@ -200,12 +210,6 @@ struct dynent : physent                         // animated characters, or chara
     }
 
     vec abovehead() { return vec(o).add(vec(0, 0, aboveeye+4)); }
-
-    void normalize_yaw(float angle)
-    {
-        while(yaw<angle-180.0f) yaw += 360.0f;
-        while(yaw>angle+180.0f) yaw -= 360.0f;
-    }
 };
 
 
