@@ -1,4 +1,5 @@
 struct vec4;
+struct vec2;
 
 struct vec
 {
@@ -12,11 +13,12 @@ struct vec
     explicit vec(int a) : x(a), y(a), z(a) {} 
     explicit vec(float a) : x(a), y(a), z(a) {} 
     vec(float a, float b, float c) : x(a), y(b), z(c) {}
-    vec(int v[3]) : x(v[0]), y(v[1]), z(v[2]) {}
-    vec(float *v) : x(v[0]), y(v[1]), z(v[2]) {}
+    explicit vec(int v[3]) : x(v[0]), y(v[1]), z(v[2]) {}
+    explicit vec(float *v) : x(v[0]), y(v[1]), z(v[2]) {}
     explicit vec(const vec4 &v);
+    explicit vec(const vec2 &v, float z = 0);
 
-    vec(float yaw, float pitch) : x(sinf(yaw)*cosf(pitch)), y(-cosf(yaw)*cosf(pitch)), z(sinf(pitch)) {}
+    vec(float yaw, float pitch) : x(-sinf(yaw)*cosf(pitch)), y(cosf(yaw)*cosf(pitch)), z(sinf(pitch)) {}
 
     float &operator[](int i)       { return v[i]; }
     float  operator[](int i) const { return v[i]; }
@@ -28,6 +30,7 @@ struct vec
 
     bool iszero() const { return x==0 && y==0 && z==0; }
     float squaredlen() const { return x*x + y*y + z*z; }
+    float dot2(const vec &o) const { return x*o.x + y*o.y; }
     float dot(const vec &o) const { return x*o.x + y*o.y + z*o.z; }
     vec &mul(const vec &o)   { x *= o.x; y *= o.y; z *= o.z; return *this; }
     vec &mul(float f)        { x *= f; y *= f; z *= f; return *this; }
@@ -37,15 +40,25 @@ struct vec
     vec &add(float f)        { x += f; y += f; z += f; return *this; }
     vec &sub(const vec &o)   { x -= o.x; y -= o.y; z -= o.z; return *this; }
     vec &sub(float f)        { x -= f; y -= f; z -= f; return *this; }
-    vec &neg()               { return mul(-1); }
+    vec &neg2()              { x = -x; y = -y; return *this; }
+    vec &neg()               { x = -x; y = -y; z = -z; return *this; }
+    vec &min(const vec &o)   { x = ::min(x, o.x); y = ::min(y, o.y); z = ::min(z, o.z); return *this; }
+    vec &max(const vec &o)   { x = ::max(x, o.x); y = ::max(y, o.y); z = ::max(z, o.z); return *this; }
+    vec &min(float f)        { x = ::min(x, f); y = ::min(y, f); z = ::min(z, f); return *this; }
+    vec &max(float f)        { x = ::max(x, f); y = ::max(y, f); z = ::max(z, f); return *this; }
+    float magnitude2() const { return sqrtf(dot2(*this)); }
     float magnitude() const  { return sqrtf(squaredlen()); }
     vec &normalize()         { div(magnitude()); return *this; }
     bool isnormalized() const { float m = squaredlen(); return (m>0.99f && m<1.01f); }
     float squaredist(const vec &e) const { return vec(*this).sub(e).squaredlen(); }
     float dist(const vec &e) const { vec t; return dist(e, t); }
     float dist(const vec &e, vec &t) const { t = *this; t.sub(e); return t.magnitude(); }
-    bool reject(const vec &o, float max) { return x>o.x+max || x<o.x-max || y>o.y+max || y<o.y-max; }
-    vec &cross(const vec &a, const vec &b) { x = a.y*b.z-a.z*b.y; y = a.z*b.x-a.x*b.z; z = a.x*b.y-a.y*b.x; return *this; }
+    bool reject(const vec &o, float r) { return x>o.x+r || x<o.x-r || y>o.y+r || y<o.y-r; }
+    template<class A, class B>
+    vec &cross(const A &a, const B &b) { x = a.y*b.z-a.z*b.y; y = a.z*b.x-a.x*b.z; z = a.x*b.y-a.y*b.x; return *this; }
+    vec &cross(const vec &o, const vec &a, const vec &b) { return cross(vec(a).sub(o), vec(b).sub(o)); }
+    float scalartriple(const vec &a, const vec &b) const { return x*(a.y*b.z-a.z*b.y) + y*(a.z*b.x-a.x*b.z) + z*(a.x*b.y-a.y*b.x); }
+    vec &reflectz(float rz) { z = 2*rz - z; return *this; }
     vec &reflect(const vec &n) { float k = 2*dot(n); x -= k*n.x; y -= k*n.y; z -= k*n.z; return *this; }
     vec &project(const vec &n) { float k = dot(n); x -= k*n.x; y -= k*n.y; z -= k*n.z; return *this; }
     vec &projectxydir(const vec &n) { if(n.z) z = -(x*n.x/n.z + y*n.y/n.z); return *this; }
@@ -53,17 +66,18 @@ struct vec
     {
         float m = squaredlen(), k = dot(n);
         projectxydir(n);
-        rescale(sqrtf(max(m - k*k, 0.0f)));
+        rescale(sqrtf(::max(m - k*k, 0.0f)));
         return *this;
     }
     vec &projectxy(const vec &n, float threshold)
     {
-        float m = squaredlen(), k = min(dot(n), threshold);
+        float m = squaredlen(), k = ::min(dot(n), threshold);
         projectxydir(n);
-        rescale(sqrtf(max(m - k*k, 0.0f)));
+        rescale(sqrtf(::max(m - k*k, 0.0f)));
         return *this;
     }
-    void lerp(const vec &a, const vec &b, float t) { x = a.x + (b.x-a.x)*t; y = a.y + (b.y-a.y)*t; z = a.z + (b.z-a.z)*t; }
+    vec &lerp(const vec &b, float t) { x += (b.x-x)*t; y += (b.y-y)*t; z += (b.z-z)*t; return *this; }
+    vec &lerp(const vec &a, const vec &b, float t) { x = a.x + (b.x-a.x)*t; y = a.y + (b.y-a.y)*t; z = a.z + (b.z-a.z)*t; return *this; }
 
     vec &rescale(float k)
     {
@@ -129,9 +143,10 @@ static inline bool htcmp(const vec &x, const vec &y)
 
 static inline uint hthash(const vec &k)
 {
-    union { int i; float f; } x, y, z;
+    union { uint i; float f; } x, y, z;
     x.f = k.x; y.f = k.y; z.f = k.z;
-    return x.i^y.i^z.i;
+    uint v = x.i^y.i^z.i;
+    return v + (v>>12);
 }
 
 struct vec4
@@ -158,16 +173,29 @@ struct vec4
     float magnitude3() const { return sqrtf(dot3(*this)); }
     vec4 &normalize() { mul(1/magnitude()); return *this; }
 
-    void lerp(const vec4 &a, const vec4 &b, float t) 
+    vec4 &lerp(const vec4 &b, float t)
+    {
+        x += (b.x-x)*t;
+        y += (b.y-y)*t;
+        z += (b.z-z)*t;
+        w += (b.w-w)*t;
+        return *this;
+    }
+    vec4 &lerp(const vec4 &a, const vec4 &b, float t) 
     { 
         x = a.x+(b.x-a.x)*t; 
         y = a.y+(b.y-a.y)*t; 
         z = a.z+(b.z-a.z)*t;
-        w = a.w+(b.w-b.w)*t;
+        w = a.w+(b.w-a.w)*t;
+        return *this;
     }
 
     vec4 &mul3(float f)      { x *= f; y *= f; z *= f; return *this; }
     vec4 &mul(float f)       { mul3(f); w *= f; return *this; }
+    vec4 &mul(const vec4 &o) { x *= o.x; y *= o.y; z *= o.z; w *= o.w; return *this; }
+    vec4 &div3(float f)      { x /= f; y /= f; z /= f; return *this; }
+    vec4 &div(float f)       { div3(f); w /= f; return *this; }
+    vec4 &div(const vec4 &o) { x /= o.x; y /= o.y; z /= o.z; w /= o.w; return *this; }
     vec4 &add(const vec4 &o) { x += o.x; y += o.y; z += o.z; w += o.w; return *this; }
     vec4 &addw(float f)      { w += f; return *this; }
     vec4 &sub(const vec4 &o) { x -= o.x; y -= o.y; z -= o.z; w -= o.w; return *this; }
@@ -180,6 +208,52 @@ struct vec4
 
 inline vec::vec(const vec4 &v) : x(v.x), y(v.y), z(v.z) {}
 
+struct vec2
+{
+    union
+    {
+        struct { float x, y; };
+        float v[2];
+    };
+
+    vec2() {}
+    vec2(float x, float y) : x(x), y(y) {}
+    explicit vec2(const vec &v) : x(v.x), y(v.y) {}
+    explicit vec2(const vec4 &v) : x(v.x), y(v.y) {}
+
+    float &operator[](int i)       { return v[i]; }
+    float  operator[](int i) const { return v[i]; }
+
+    bool operator==(const vec2 &o) const { return x == o.x && y == o.y; }
+    bool operator!=(const vec2 &o) const { return x != o.x || y != o.y; }
+
+    float dot(const vec2 &o) const  { return x*o.x + y*o.y; }
+    float squaredlen() const { return dot(*this); }
+    float magnitude() const  { return sqrtf(squaredlen()); }
+    vec2 &normalize() { mul(1/magnitude()); return *this; }
+    float cross(const vec2 &o) const { return x*o.y - y*o.x; }
+
+    vec2 &mul(float f)       { x *= f; y *= f; return *this; }
+    vec2 &add(const vec2 &o) { x += o.x; y += o.y; return *this; }
+    vec2 &sub(const vec2 &o) { x -= o.x; y -= o.y; return *this; }
+    vec2 &neg()              { x = -x; y = -y; return *this; }
+};
+
+inline vec::vec(const vec2 &v, float z) : x(v.x), y(v.y), z(z) {}
+
+static inline bool htcmp(const vec2 &x, const vec2 &y)
+{
+    return x == y;
+}
+
+static inline uint hthash(const vec2 &k)
+{
+    union { uint i; float f; } x, y;
+    x.f = k.x; y.f = k.y;
+    uint v = x.i^y.i;
+    return v + (v>>12);
+}
+
 struct matrix3x3;
 struct matrix3x4;
 
@@ -195,32 +269,31 @@ struct quat : vec4
         y = s*axis.y;
         z = s*axis.z;
     }
+    explicit quat(const vec &v)
+    {
+        x = v.x;
+        y = v.y;
+        z = v.z;
+        restorew();
+    }
     explicit quat(const matrix3x3 &m) { convertmatrix(m); }
     explicit quat(const matrix3x4 &m) { convertmatrix(m); }
 
     void restorew() { w = 1.0f-x*x-y*y-z*z; w = w<0 ? 0 : -sqrtf(w); }
     
-    void add(const vec4 &o) { vec4::add(o); }
-    void sub(const vec4 &o) { vec4::sub(o); }
-    void mul(float k) { vec4::mul(k); }
+    quat &add(const vec4 &o) { vec4::add(o); return *this; }
+    quat &sub(const vec4 &o) { vec4::sub(o); return *this; }
+    quat &mul(float k) { vec4::mul(k); return *this; }
 
-    void mul(const quat &p, const quat &o)
+    quat &mul(const quat &p, const quat &o)
     {
         x = p.w*o.x + p.x*o.w + p.y*o.z - p.z*o.y;
         y = p.w*o.y - p.x*o.z + p.y*o.w + p.z*o.x;
         z = p.w*o.z + p.x*o.y - p.y*o.x + p.z*o.w;
         w = p.w*o.w - p.x*o.x - p.y*o.y - p.z*o.z;
+        return *this;
     }
-    void mul(const quat &o) { mul(quat(*this), o); }
-
-    void mul(const quat &p, const vec &o)
-    {
-        x =  p.w*o.x + p.y*o.z - p.z*o.y;
-        y =  p.w*o.y - p.x*o.z + p.z*o.x;
-        z =  p.w*o.z + p.x*o.y - p.y*o.x;
-        w = -p.x*o.x - p.y*o.y - p.z*o.z;
-    }
-    void mul(const vec &o) { mul(quat(*this), o); }
+    quat &mul(const quat &o) { return mul(quat(*this), o); }
 
     quat &invert() { neg3(); return *this; }
 
@@ -235,38 +308,14 @@ struct quat : vec4
         else { angle = 0; axis = vec(0, 0, 1); }
     }
 
-    void slerp(const quat &from, const quat &to, float t)
-    {
-        float cosomega = from.dot(to), fromk, tok = 1;
-        if(cosomega < 0) { cosomega = -cosomega; tok = -1; }
-
-        if(cosomega > 1 - 1e-6) { fromk = 1-t; tok *= t; }
-        else
-        {
-            float omega = acosf(cosomega), recipsinomega = 1/sinf(omega);
-            fromk = sinf((1-t)*omega)*recipsinomega;
-            tok *= sinf(t*omega)*recipsinomega;
-        }
-
-        loopi(4) v[i] = from[i]*fromk + to[i]*tok;
-    }
-
     vec rotate(const vec &v) const
     {
-        vec t1, t2;
-        t1.cross(vec(*this), v);
-        t2.cross(vec(*this), t1);
-        t1.mul(w).add(t2).mul(2).add(v);
-        return t1;
-#if 0
-        quat inv(*this);
-        inv.invert();
-        inv.normalize();
-        quat tmp;
-        tmp.mul(*this, v);
-        tmp.mul(inv);
-        return vec(tmp.x, tmp.y, tmp.z);
-#endif
+        return vec().cross(*this, vec().cross(*this, v).add(vec(v).mul(w))).mul(2).add(v);
+    }
+
+    vec invertedrotate(const vec &v) const
+    {
+        return vec().cross(*this, vec().cross(*this, v).sub(vec(v).mul(w))).mul(2).add(v);
     }
 
     template<class M>
@@ -327,45 +376,33 @@ struct dualquat
     dualquat &mul(float k) { real.mul(k); dual.mul(k); return *this; }
     dualquat &add(const dualquat &d) { real.add(d.real); dual.add(d.dual); return *this; }
 
-    void lerp(const dualquat &from, const dualquat &to, float t)
+    dualquat &lerp(const dualquat &to, float t)
     {
-        float a = 1-t, b = from.real.dot(to.real)<0 ? -t : t;
-        loopi(4)
-        {
-            real[i] = from.real[i]*a + to.real[i]*b;
-            dual[i] = from.dual[i]*a + to.dual[i]*b;
-        }
+        float k = real.dot(to.real) < 0 ? -t : t;
+        real.mul(1-t).add(vec4(to.real).mul(k));
+        dual.mul(1-t).add(vec4(to.dual).mul(k));
+        return *this;
+    }
+    dualquat &lerp(const dualquat &from, const dualquat &to, float t)
+    {
+        float k = from.real.dot(to.real) < 0 ? -t : t;
+        (real = from.real).mul(1-t).add(vec4(to.real).mul(k));
+        (dual = from.dual).mul(1-t).add(vec4(to.dual).mul(k));
+        return *this;
     }
 
     dualquat &invert()
     {
-        float rr = real.squaredlen();
-        if(rr > 0)
-        {
-            float invrr = 1/rr,
-                  invrd = -2*real.dot(dual)*invrr*invrr;
-
-            dual.mul3(-invrr);
-            dual.w *= invrr;
-            quat tmp(real);
-            tmp.mul3(-invrd);
-            tmp.w *= invrd;
-            dual.add(tmp);
-
-            real.mul3(-invrr);
-            real.w *= invrr;
-        }
-        else { real = dual = quat(0, 0, 0, 0); }
+        real.invert();
+        dual.invert();
+        dual.sub(quat(real).mul(2*real.dot(dual)));
         return *this;
     }
     
     void mul(const dualquat &p, const dualquat &o)
     {
         real.mul(p.real, o.real);
-        dual.mul(p.real, o.dual);
-        quat tmp;
-        tmp.mul(p.dual, o.real);
-        dual.add(tmp);
+        dual.mul(p.real, o.dual).add(quat().mul(p.dual, o.real));
     }       
     void mul(const dualquat &o) { mul(dualquat(*this), o); }    
   
@@ -375,6 +412,16 @@ struct dualquat
         dual.mul(quat(q).invert(), quat(dual));
     }
 
+    void mulorient(const quat &q, const dualquat &base)
+    {
+        quat trans;
+        trans.mul(base.dual, quat(base.real).invert());
+        dual.mul(quat(q).invert(), quat(real).mul(trans).add(dual));
+
+        real.mul(q, quat(real));
+        dual.add(quat().mul(real, trans.invert())).sub(quat(real).mul(2*base.real.dot(base.dual)));
+    }
+
     void normalize()
     {
         float invlen = 1/real.magnitude();
@@ -406,33 +453,34 @@ struct dualquat
 
     void accumulate(const dualquat &d, float k)
     {
-        if(real.dot(d.real) < 0)
-        {
-            real.sub(vec4(d.real).mul(k));
-            dual.sub(vec4(d.dual).mul(k));
-        }
-        else
-        {
-            real.add(vec4(d.real).mul(k));
-            dual.add(vec4(d.dual).mul(k));
-        }
+        if(real.dot(d.real) < 0) k = -k;
+        real.add(vec4(d.real).mul(k));
+        dual.add(vec4(d.dual).mul(k));
     }
 
     vec transform(const vec &v) const
     {
-        vec t1, t2;
-        t1.cross(vec(real), v);
-        t1.add(vec(v).mul(real.w));
-        t2.cross(vec(real), t1);
+        return vec().cross(real, vec().cross(real, v).add(vec(v).mul(real.w)).add(vec(dual))).add(vec(dual).mul(real.w)).sub(vec(real).mul(dual.w)).mul(2).add(v);
+    }
 
-        vec t3;
-        t3.cross(vec(real), vec(dual));
-        t3.add(vec(dual).mul(real.w));
-        t3.sub(vec(real).mul(dual.w));
+    vec transposedtransform(const vec &v) const
+    {
+        return dualquat(*this).invert().transform(v);
+    }
 
-        t2.add(t3).mul(2).add(v);
+    vec transformnormal(const vec &v) const
+    {
+        return real.rotate(v);
+    }
 
-        return t2;
+    vec transposedtransformnormal(const vec &v) const
+    {
+        return real.invertedrotate(v);
+    }
+
+    vec gettranslation() const
+    {
+        return vec().cross(real, dual).add(vec(dual).mul(real.w)).sub(vec(real).mul(dual.w)).mul(2);
     }
 };
 
@@ -495,33 +543,43 @@ struct matrix3x3
         c = vec(axis.x*axis.z*(1-ck)-axis.y*sk, axis.y*axis.z*(1-ck)+axis.x*sk, axis.z*axis.z*(1-ck)+ck);
     }
 
-    void calcangleaxis(float &angle, vec &axis)
+    bool calcangleaxis(float &angle, vec &axis, float threshold = 1e-9f)
     {
         angle = acosf(clamp(0.5f*(a.x + b.y + c.z - 1), -1.0f, 1.0f));
 
 		if(angle <= 0) axis = vec(0, 0, 1);
-        else if(angle < M_PI) axis = vec(c.y - b.z, a.z - c.x, b.x - a.y).normalize();
+        else if(angle < M_PI) 
+        {
+            axis = vec(c.y - b.z, a.z - c.x, b.x - a.y);
+            float r = axis.magnitude();
+            if(r <= threshold) return false;
+            axis.mul(1/r);
+        }
         else if(a.x >= b.y && a.x >= c.z)
         {
-            float r = sqrtf(1 + a.x - b.y - c.z), inv = 1/r;
+            float r = sqrtf(1 + a.x - b.y - c.z);
+            if(r <= threshold) return false;
             axis.x = 0.5f*r;
-            axis.y = a.y*inv;
-            axis.z = a.z*inv;
+            axis.y = a.y/r;
+            axis.z = a.z/r;
         }
         else if(b.y >= c.z)
         {
-            float r = sqrtf(1 + b.y - a.x - c.z), inv = 1/r;
+            float r = sqrtf(1 + b.y - a.x - c.z);
+            if(r <= threshold) return false;
             axis.y = 0.5f*r;
-            axis.x = a.y*inv;
-            axis.z = b.z*inv;
+            axis.x = a.y/r;
+            axis.z = b.z/r;
         }
         else
         {
-            float r = sqrtf(1 + b.y - a.x - c.z), inv = 1/r;
+            float r = sqrtf(1 + b.y - a.x - c.z);
+            if(r <= threshold) return false;
             axis.z = 0.5f*r;
-            axis.x = a.z*inv;
-            axis.y = b.z*inv;
+            axis.x = a.z/r;
+            axis.y = b.z/r;
         }
+        return true;
     }
 
     vec transform(const vec &o) const { return vec(a.dot(o), b.dot(o), c.dot(o)); }
@@ -533,6 +591,22 @@ struct matrix3x3
     }
 };
 
+struct matrix2x3
+{
+    vec a, b;
+
+    matrix2x3() {}
+    matrix2x3(const vec &a, const vec &b) : a(a), b(b) {}
+
+    vec2 transform(const vec &o) const { return vec2(a.dot(o), b.dot(o)); }
+    vec transposedtransform(const vec2 &o) const
+    {
+        return vec(a.x*o.x + b.x*o.y,
+                   a.y*o.x + b.y*o.y,
+                   a.z*o.x + b.z*o.y);
+    }
+};
+
 struct matrix3x4
 {
     vec4 a, b, c;
@@ -544,21 +618,23 @@ struct matrix3x4
     {}
     matrix3x4(const dualquat &d)
     {
-        float x = d.real.x, y = d.real.y, z = d.real.z, w = d.real.w, 
-              ww = w*w, xx = x*x, yy = y*y, zz = z*z,
-              xy = x*y, xz = x*z, yz = y*z,
-              wx = w*x, wy = w*y, wz = w*z;
-        a = vec4(ww + xx - yy - zz, 2*(xy - wz), 2*(xz + wy),
-            -2*(d.dual.w*x - d.dual.x*w + d.dual.y*z - d.dual.z*y));
-        b = vec4(2*(xy + wz), ww + yy - xx - zz, 2*(yz - wx),
-            -2*(d.dual.w*y - d.dual.x*z - d.dual.y*w + d.dual.z*x));
-        c = vec4(2*(xz - wy), 2*(yz + wx), ww + zz - xx - yy,
-            -2*(d.dual.w*z + d.dual.x*y - d.dual.y*x - d.dual.z*w));
+        vec4 r = vec4(d.real).mul(1/d.real.squaredlen()), rr = vec4(r).mul(d.real);
+        r.mul(2);
+        float xy = r.x*d.real.y, xz = r.x*d.real.z, yz = r.y*d.real.z,
+              wx = r.w*d.real.x, wy = r.w*d.real.y, wz = r.w*d.real.z;
+        a = vec4(rr.w + rr.x - rr.y - rr.z, xy - wz, xz + wy,
+            -(d.dual.w*r.x - d.dual.x*r.w + d.dual.y*r.z - d.dual.z*r.y));
+        b = vec4(xy + wz, rr.w + rr.y - rr.x - rr.z, yz - wx,
+            -(d.dual.w*r.y - d.dual.x*r.z - d.dual.y*r.w + d.dual.z*r.x));
+        c = vec4(xz - wy, yz + wx, rr.w + rr.z - rr.x - rr.y,
+            -(d.dual.w*r.z + d.dual.x*r.y - d.dual.y*r.x - d.dual.z*r.w));
+    }
 
-        float invrr = 1/d.real.dot(d.real);
-        a.mul(invrr);
-        b.mul(invrr);
-        c.mul(invrr);
+    void mul(float k)
+    {
+        a.mul(k);
+        b.mul(k);
+        c.mul(k);
     }
 
     void scale(float k)
@@ -567,7 +643,7 @@ struct matrix3x4
         b.mul(k);
         c.mul(k);
     }
-
+    
     void translate(const vec &p)
     {
         a.w += p.x;
@@ -575,6 +651,13 @@ struct matrix3x4
         c.w += p.z;
     }
 
+    void transformedtranslate(const vec &p, float scale = 1)
+    {
+        a.w += a.dot3(p)*scale;
+        b.w += b.dot3(p)*scale;
+        c.w += c.dot3(p)*scale;
+    }
+
     void accumulate(const matrix3x4 &m, float k)
     {
         a.add(vec4(m.a).mul(k));
@@ -589,14 +672,17 @@ struct matrix3x4
         c.mul3(1/c.magnitude3());
     }
 
+    void lerp(const matrix3x4 &to, float t)
+    {
+        a.lerp(to.a, t);
+        b.lerp(to.b, t);
+        c.lerp(to.c, t);
+    }
     void lerp(const matrix3x4 &from, const matrix3x4 &to, float t)
     {
-        loopi(4)
-        {
-            a[i] += from.a[i] + (to.a[i]-from.a[i])*t;
-            b[i] += from.b[i] + (to.b[i]-from.b[i])*t;
-            c[i] += from.c[i] + (to.c[i]-from.c[i])*t;
-        }
+        a.lerp(from.a, to.a, t);
+        b.lerp(from.b, to.b, t);
+        c.lerp(from.c, to.c, t);
     }
 
     void identity()
@@ -614,6 +700,13 @@ struct matrix3x4
     }
     void mul(const matrix3x4 &n) { mul(*this, n); }
 
+    void mul(const matrix3x3 &rot, const vec &trans, const matrix3x4 &o)
+    {
+        a = vec4(o.a).mul(rot.a.x).add(vec4(o.b).mul(rot.a.y)).add(vec4(o.c).mul(rot.a.z)).addw(trans.x);
+        b = vec4(o.a).mul(rot.b.x).add(vec4(o.b).mul(rot.b.y)).add(vec4(o.c).mul(rot.b.z)).addw(trans.y);
+        c = vec4(o.a).mul(rot.c.x).add(vec4(o.b).mul(rot.c.y)).add(vec4(o.c).mul(rot.c.z)).addw(trans.z);
+    }
+
     void mulorient(const matrix3x3 &m)
     {
         vec ra = vec(a).mul(m.a.x).add(vec(b).mul(m.a.y)).add(vec(c).mul(m.a.z)),
@@ -624,6 +717,13 @@ struct matrix3x4
         c.setxyz(rc);
     }
 
+    void mulorient(const matrix3x3 &m, const dualquat &base)
+    {
+        vec trans = transformnormal(base.gettranslation());
+        translate(trans.sub(m.transform(trans)));
+        mulorient(m);
+    }
+
     void transpose(const matrix3x4 &o)
     {
         a = vec4(o.a.x, o.b.x, o.c.x, -(o.a.x*o.a.w + o.b.x*o.b.w + o.c.x*o.c.w));
@@ -734,6 +834,7 @@ struct plane : vec
 
     plane() {}
     plane(const vec &c, float off) : vec(c), offset(off) {} 
+    plane(const vec4 &p) : vec(p), offset(p.w) {}
     plane(int d, float off)
     {
         x = y = z = 0.0f;
@@ -767,6 +868,20 @@ struct plane : vec
         return true;
     }
 
+    plane &reflectz(float rz)
+    {
+        offset += 2*rz*z;
+        z = -z;
+        return *this; 
+    }
+
+    plane &invert()
+    {
+        neg();
+        offset = -offset;
+        return *this;
+    }
+
     plane &scale(float k)
     {
         mul(k);
@@ -779,6 +894,14 @@ struct plane : vec
         return *this;
     }
 
+    plane &normalize()
+    {
+        float mag = magnitude();
+        div(mag);
+        offset /= mag; 
+        return *this;
+    }
+
     float zintersect(const vec &p) const { return -(x*p.x+y*p.y+offset)/z; }
     float zdist(const vec &p) const { return p.z-zintersect(p); }
 };
@@ -867,6 +990,10 @@ struct ivec
     ivec &sub(const ivec &v) { x -= v.x; y -= v.y; z -= v.z; return *this; }
     ivec &mask(int n) { x &= n; y &= n; z &= n; return *this; }
     ivec &neg() { return mul(-1); }
+    ivec &min(const ivec &o) { x = ::min(x, o.x); y = ::min(y, o.y); z = ::min(z, o.z); return *this; }
+    ivec &max(const ivec &o) { x = ::max(x, o.x); y = ::max(y, o.y); z = ::max(z, o.z); return *this; }
+    ivec &min(int n) { x = ::min(x, n); y = ::min(y, n); z = ::min(z, n); return *this; }
+    ivec &max(int n) { x = ::max(x, n); y = ::max(y, n); z = ::max(z, n); return *this; }
     ivec &cross(const ivec &a, const ivec &b) { x = a.y*b.z-a.z*b.y; y = a.z*b.x-a.x*b.z; z = a.x*b.y-a.y*b.x; return *this; }
     int dot(const ivec &o) const { return x*o.x + y*o.y + z*o.z; }
     float dist(const plane &p) const { return x*p.x + y*p.y + z*p.z + p.offset; }
@@ -882,35 +1009,6 @@ static inline uint hthash(const ivec &k)
     return k.x^k.y^k.z;
 }  
 
-struct svec
-{
-    union
-    {
-        struct { ushort x, y, z; };
-        ushort v[3];
-    };
-
-    svec() {}
-    svec(ushort x, ushort y, ushort z) : x(x), y(y), z(z) {}
-
-    ushort &operator[](int i)       { return v[i]; }
-    ushort  operator[](int i) const { return v[i]; }
-
-    bool operator==(const svec &v) const { return x==v.x && y==v.y && z==v.z; }
-    bool operator!=(const svec &v) const { return x!=v.x || y!=v.y || z!=v.z; }
-
-    svec &add(const svec &o) { x += o.x; y += o.y; z += o.z; return *this; }
-    svec &add(const ivec &o) { x += o.x; y += o.y; z += o.z; return *this; }
-    svec &add(int n) { x += n; y += n; z += n; return *this; }
-    svec &sub(const svec &o) { x -= o.x; y -= o.y; z -= o.z; return *this; }
-    svec &sub(const ivec &o) { x -= o.x; y -= o.y; z -= o.z; return *this; }
-    svec &sub(int n) { x -= n; y -= n; z -= n; return *this; }
-    svec &mul(int f) { x *= f; y *= f; z *= f; return *this; }
-    svec &div(int f) { x /= f; y /= f; z /= f; return *this; }
-
-    bool iszero() const { return x==0 && y==0 && z==0; }
-};
-
 struct bvec
 {
     union
@@ -932,6 +1030,10 @@ struct bvec
     bool iszero() const { return x==0 && y==0 && z==0; }
 
     vec tovec() const { return vec(x*(2.0f/255.0f)-1.0f, y*(2.0f/255.0f)-1.0f, z*(2.0f/255.0f)-1.0f); }
+
+    void lerp(const bvec &a, const bvec &b, float t) { x = uchar(a.x + (b.x-a.x)*t); y = uchar(a.y + (b.y-a.y)*t); z = uchar(a.z + (b.z-a.z)*t); }
+
+    void flip() { x -= 128; y -= 128; z -= 128; }
 };
 
 struct glmatrixf
@@ -939,13 +1041,20 @@ struct glmatrixf
     float v[16];
 
     glmatrixf() {}
+    glmatrixf(const float *m) { memcpy(v, m, sizeof(v)); }
+    glmatrixf(const vec4 &a, const vec4 &b, const vec4 &c, const vec4 &d = vec4(0, 0, 0, 1))
+    {
+        v[0]  = a.x; v[1] =  b.x; v[2]  = c.x; v[3]  = d.x;
+        v[4]  = a.y; v[5] =  b.y; v[6]  = c.y; v[7]  = d.y;
+        v[8]  = a.z; v[9] =  b.z; v[10] = c.z; v[11] = d.z;
+        v[12] = a.w; v[13] = b.w; v[14] = c.w; v[15] = d.w;
+    }
     glmatrixf(const matrix3x4 &m)
     {
-        v[0] = m.a.x; v[1] = m.b.x; v[2] = m.c.x;
-        v[4] = m.a.y; v[5] = m.b.y; v[6] = m.c.y;
-        v[8] = m.a.z; v[9] = m.b.z; v[10] = m.c.z;
-        v[12] = m.a.w; v[13] = m.b.w; v[14] = m.c.w;
-        v[3] = v[7] = v[11] = 0.0f; v[15] = 1.0f;
+        v[0]  = m.a.x; v[1] =  m.b.x; v[2]  = m.c.x; v[3]  = 0.0f;
+        v[4]  = m.a.y; v[5] =  m.b.y; v[6]  = m.c.y; v[7]  = 0.0f;
+        v[8]  = m.a.z; v[9] =  m.b.z; v[10] = m.c.z; v[11] = 0.0f;
+        v[12] = m.a.w; v[13] = m.b.w; v[14] = m.c.w; v[15] = 1.0f;
     }
 
     float operator[](int i) const { return v[i]; }
@@ -1030,6 +1139,7 @@ struct glmatrixf
     {
         mul(x.v, y.v);
     }
+    void mul(const glmatrixf &y) { mul(glmatrixf(*this), y); }
 
     void identity()
     {
@@ -1055,6 +1165,18 @@ struct glmatrixf
         translate(o.x, o.y, o.z);
     }
 
+    void transformedtranslate(float x, float y, float z, float scale = 1)
+    {
+        v[12] += (x*v[0] + y*v[4] + z*v[8])*scale;
+        v[13] += (x*v[1] + y*v[5] + z*v[9])*scale;
+        v[14] += (x*v[2] + y*v[6] + z*v[10])*scale;
+    }
+
+    void transformedtranslate(const vec &p, float scale = 1)
+    {
+        transformedtranslate(p.x, p.y, p.z, scale);
+    }
+
     void scale(float x, float y, float z)
     {
         v[0] *= x; v[1] *= x; v[2] *= x; v[3] *= x;
@@ -1064,8 +1186,11 @@ struct glmatrixf
 
     void reflectz(float z)
     {
+        v[12] += 2*z*v[8];
+        v[13] += 2*z*v[9];
+        v[14] += 2*z*v[10];
+        v[15] += 2*z*v[11];
         v[8] = -v[8]; v[9] = -v[9]; v[10] = -v[10]; v[11] = -v[11];
-        v[14] += 2*z;
     }
 
     void projective(float zscale = 0.5f, float zoffset = 0.5f)
@@ -1131,26 +1256,46 @@ struct glmatrixf
         invertnormal(p);
     }
 
-    template<class T> float transformx(const T &p) const
+    float transformx(const vec &p) const
     {
         return p.x*v[0] + p.y*v[4] + p.z*v[8] + v[12];
     }
 
-    template<class T> float transformy(const T &p) const
+    float transformy(const vec &p) const
     {
         return p.x*v[1] + p.y*v[5] + p.z*v[9] + v[13];
     }
 
-    template<class T> float transformz(const T &p) const
+    float transformz(const vec &p) const
     {
         return p.x*v[2] + p.y*v[6] + p.z*v[10] + v[14];
     }
 
-    template<class T> float transformw(const T &p) const
+    float transformw(const vec &p) const
     {
         return p.x*v[3] + p.y*v[7] + p.z*v[11] + v[15];
     }
 
+    float transformx(const vec4 &p) const
+    {
+        return p.x*v[0] + p.y*v[4] + p.z*v[8] + p.w*v[12];
+    }
+
+    float transformy(const vec4 &p) const
+    {
+        return p.x*v[1] + p.y*v[5] + p.z*v[9] + p.w*v[13];
+    }
+
+    float transformz(const vec4 &p) const
+    {
+        return p.x*v[2] + p.y*v[6] + p.z*v[10] + p.w*v[14];
+    }
+
+    float transformw(const vec4 &p) const
+    {
+        return p.x*v[3] + p.y*v[7] + p.z*v[11] + p.w*v[15];
+    }
+
     template<class T> void transform(const T &in, vec &out) const
     {
         out.x = transformx(in);
@@ -1166,7 +1311,27 @@ struct glmatrixf
         out.w = transformw(in);
     }
 
-    void transposetransform(const plane &in, plane &out) const
+    template<class T> vec perspectivetransform(const T &in) const
+    {
+        return vec(transformx(in), transformy(in), transformz(in)).div(transformw(in));
+    }
+
+    void transposedtransform(const vec &in, vec &out) const
+    {
+        vec p(in.x - v[12], in.y - v[13], in.z - v[14]);
+        out.x = p.x*v[0] + p.y*v[1] + p.z*v[2];
+        out.y = p.x*v[4] + p.y*v[5] + p.z*v[6];
+        out.z = p.x*v[8] + p.y*v[9] + p.z*v[10];
+    }
+
+    void transposedtransformnormal(const vec &in, vec &out) const
+    {
+        out.x = in.x*v[0] + in.y*v[1] + in.z*v[2];
+        out.y = in.x*v[4] + in.y*v[5] + in.z*v[6];
+        out.z = in.x*v[8] + in.y*v[9] + in.z*v[10];
+    }
+
+    void transposedtransform(const plane &in, plane &out) const
     {
         out.x      = in.x*v[0] + in.y*v[1] + in.z*v[2] + in.offset*v[3];
         out.y      = in.x*v[4] + in.y*v[5] + in.z*v[6] + in.offset*v[7];
@@ -1184,6 +1349,9 @@ struct glmatrixf
         return vec(v[12], v[13], v[14]);
     }
 
+    vec4 getrow(int i) const { return vec4(v[i], v[i+4], v[i+8], v[i+12]); }
+    vec4 getcolumn(int i) const { i *= 4; return vec4(v[i], v[i+1], v[i+2], v[i+3]); }
+
     float determinant() const;
     void adjoint(const glmatrixf &m);
     bool invert(const glmatrixf &m, float mindet = 1.0e-10f);
