@@ -210,7 +210,7 @@ static void mountzip(ziparchive &arch, vector<zipfile> &files, const char *mount
 {
     string packagesdir = "packages/";
     path(packagesdir);
-    int striplen = stripdir ? strlen(stripdir) : 0;
+    int striplen = stripdir ? (int)strlen(stripdir) : 0;
     if(!mountdir && !stripdir) loopv(files)
     {
         zipfile &f = files[i];
@@ -264,7 +264,7 @@ bool addzip(const char *name, const char *mount = NULL, const char *strip = NULL
     string pname;
     copystring(pname, name);
     path(pname);
-    int plen = strlen(pname);
+    int plen = (int)strlen(pname);
     if(plen < 4 || !strchr(&pname[plen-4], '.')) concatstring(pname, ".zip");
 
     ziparchive *exists = findzip(pname);
@@ -301,7 +301,11 @@ bool addzip(const char *name, const char *mount = NULL, const char *strip = NULL
      
 bool removezip(const char *name)
 {
-    const char *pname = path(name, true);
+    string pname;
+    copystring(pname, name);
+    path(pname);
+    int plen = (int)strlen(pname);
+    if(plen < 4 || !strchr(&pname[plen-4], '.')) concatstring(pname, ".zip");
     ziparchive *exists = findzip(pname);
     if(!exists)
     {
@@ -331,8 +335,9 @@ struct zipstream : stream
     z_stream zfile;
     uchar *buf;
     int reading;
+    bool ended;
 
-    zipstream() : arch(NULL), info(NULL), buf(NULL), reading(-1)
+    zipstream() : arch(NULL), info(NULL), buf(NULL), reading(-1), ended(false)
     {
         zfile.zalloc = NULL;
         zfile.zfree = NULL;
@@ -356,9 +361,8 @@ struct zipstream : stream
             if(fseek(arch->data, reading, SEEK_SET) >= 0) arch->owner = this;
             else return;
         }
-        else if(ftell(arch->data) != reading) *(int *)0 = 0;
         uint remaining = info->offset + info->compressedsize - reading;
-        int n = arch->owner == this ? fread(zfile.next_in + zfile.avail_in, 1, min(size, remaining), arch->data) : 0;
+        int n = arch->owner == this ? (int)fread(zfile.next_in + zfile.avail_in, 1, min(size, remaining), arch->data) : 0;
         zfile.avail_in += n;
         reading += n;
     }
@@ -379,6 +383,7 @@ struct zipstream : stream
         arch = a;
         info = f;
         reading = f->offset;
+        ended = false;
         if(f->compressedsize) buf = new uchar[BUFSIZE];
         return true;
     }
@@ -401,7 +406,7 @@ struct zipstream : stream
     }
 
     long size() { return info->size; }
-    bool end() { return reading < 0; }
+    bool end() { return reading < 0 || ended; }
     long tell() { return reading >= 0 ? (info->compressedsize ? zfile.total_out : reading - info->offset) : -1; }
 
     bool seek(long pos, int whence)
@@ -421,6 +426,7 @@ struct zipstream : stream
             if(fseek(arch->data, pos, SEEK_SET) < 0) return false;
             arch->owner = this;
             reading = pos;
+            ended = false;
             return true;
         }
  
@@ -439,6 +445,7 @@ struct zipstream : stream
             zfile.avail_in = 0;
             zfile.total_in = info->compressedsize; 
             arch->owner = NULL;
+            ended = false;
             return true;
         }
 
@@ -465,10 +472,11 @@ struct zipstream : stream
         while(pos > 0)
         {
             int skipped = min(pos, (long)sizeof(skip));
-            if(read(skip, skipped) != skipped) { stopreading(); return false; }
+            if(read(skip, skipped) != skipped) return false;
             pos -= skipped;
         }
 
+        ended = false;
         return true;
     }
 
@@ -484,9 +492,9 @@ struct zipstream : stream
                 arch->owner = this;
             }
               
-            int n = fread(buf, 1, min(len, int(info->size + info->offset - reading)), arch->data);
+            int n = (int)fread(buf, 1, min(len, int(info->size + info->offset - reading)), arch->data);
             reading += n;
-            if(n < len) stopreading();
+            if(n < len) ended = true;
             return n;
         }
 
@@ -498,10 +506,14 @@ struct zipstream : stream
             int err = inflate(&zfile, Z_NO_FLUSH);
             if(err != Z_OK) 
             {
+                if(err == Z_STREAM_END) ended = true;
+                else
+                {
 #ifndef STANDALONE
-                if(err != Z_STREAM_END && dbgzip) conoutf(CON_DEBUG, "inflate error: %s", err);
+                    if(dbgzip) conoutf(CON_DEBUG, "inflate error: %s", err);
 #endif
-                stopreading(); 
+                    stopreading(); 
+                }
                 break; 
             }
         }
@@ -526,7 +538,7 @@ stream *openzipfile(const char *name, const char *mode)
 
 int listzipfiles(const char *dir, const char *ext, vector<char *> &files)
 {
-    int extsize = ext ? (int)strlen(ext)+1 : 0, dirsize = strlen(dir), dirs = 0;
+    int extsize = ext ? (int)strlen(ext)+1 : 0, dirsize = (int)strlen(dir), dirs = 0;
     loopvrev(archives)
     {
         ziparchive *arch = archives[i];
